@@ -3,7 +3,7 @@ package Tk::More;
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = sprintf("%d.%02d", q$Revision: 5.3 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 5.8 $ =~ /(\d+)\.(\d+)/);
 
 use Tk qw(Ev);
 use Tk::Derived;
@@ -99,6 +99,7 @@ sub Populate {
 		   'Search'    => 'SELF',
 		   'ShowMatch' => 'SELF',
 		   'Load'      => 'SELF',
+		   'LoadFH'    => 'SELF',
 		   'AddQuitBindings' => 'SELF',
 		  );
 
@@ -196,25 +197,10 @@ sub ShowMatch {
 sub Load
 {
  my ($text,$file,%args) = @_;
- my $encoding = delete $args{-encoding};
- die "Unhandled arguments: " . join(" ", %args) if %args;
  if (open(FILE,"<$file"))
   {
-   if ($encoding)
-    {
-     binmode FILE, ":encoding($encoding)";
-    }
-   $text->MainWindow->Busy;
-   $text->SUPER::delete('1.0','end');
-   #yy delete $text->{UNDO};
-   while (<FILE>)
-    {
-     $text->SUPER::insert('end',$_);
-    }
+   $text->LoadFH(\*FILE,%args);
    close(FILE);
-   #yy $text->{FILE} = $file;
-   $text->markSet('insert', '@1,0');
-   $text->MainWindow->Unbusy;
   }
  else
   {
@@ -223,6 +209,27 @@ sub Load
   }
 }
 
+sub LoadFH
+{
+ my ($text,$fh,%args) = @_;
+ my $encoding = delete $args{-encoding};
+ die "Unhandled arguments: " . join(" ", %args) if %args;
+ if ($encoding)
+  {
+   binmode $fh, ":encoding($encoding)";
+  }
+ $text->MainWindow->Busy;
+ $text->SUPER::delete('1.0','end');
+ #yy delete $text->{UNDO};
+ while (<$fh>)
+  {
+   $text->SUPER::insert('end',$_);
+  }
+ #yy $text->{FILE} = $file;
+ $text->markSet('insert', '@1,0');
+ $text->MainWindow->Unbusy;
+}
+
 # search_text copied from demo search.pl (modified)
 sub search_text {
 
@@ -437,7 +444,7 @@ then assume no encoding (which is equivalent to iso-8859-1).
 
 =back
 
-=item AddQuitBinding
+=item AddQuitBindings
 
 Convenience method to add the bindinds Key-q and Control-Key-q to
 close the Toplevel window containing this More widget.
