@@ -4,8 +4,8 @@ use Tk ();
 use Tk::Toplevel;
 
 use vars qw($VERSION $DIST_VERSION @ISA);
-$VERSION = sprintf("%d.%02d", q$Revision: 5.25 $ =~ /(\d+)\.(\d+)/);
-$DIST_VERSION = "0.9939";
+$VERSION = '5.38';
+$DIST_VERSION = '0.9940';
 
 @ISA = qw(Tk::Toplevel);
 
@@ -122,10 +122,6 @@ EOF
      '-accelerator' => 'Ctrl+N',
      '-command' => ['newwindow',$w,$p],
     ],
-    [Button => $compound->('~Reload', "actreload"),
-     '-accelerator' => 'Ctrl+R',
-     '-command' => ['reload',$p],
-    ],
     [Button => $compound->('~Edit', "edit"),
      '-command' => ['edit',$p],
     ],
@@ -170,6 +166,15 @@ EOF
      '-command' => [$w, 'zoom_out'],
     ],
     '-',
+    [Button => $compound->('~Reload', "actreload"),
+     '-accelerator' => 'Ctrl+R',
+     '-command' => ['reload',$p],
+    ],
+    [Button => $compound->("~View source"),
+     # unfortunately Tk::More already uses Ctrl+U
+     '-command' => ['view_source',$p],
+    ],
+    '-',
     [Button => $compound->('Pod on ~search.cpan.org'),
      '-command' => sub {
 	 require Tk::Pod::Util;
@@ -328,9 +333,9 @@ EOF
   $w->bind($path, "<Control-plus>" => [$w, 'zoom_in']);
   $w->bind($path, "<F3>" => [$w,'openfile']);
   $w->bind($path, "<Control-o>" => [$w,'openpod',$p]);
-  $w->bind($path, "<Control-n>" => [$w,'newwindow',$p]);
+  $w->bind($path, "<Control-n>" => sub { $w->newwindow($p); Tk->break });
   $w->bind($path, "<Control-r>" => [$p, 'reload']);
-  $w->bind($path, "<Control-p>" => [$p, 'Print']);
+  $w->bind($path, "<Control-p>" => sub { $p->Print; Tk->break });
   $w->bind($path, "<Print>"     => [$p, 'Print']);
   $w->bind($path, "<Control-w>" => [$w, 'quit']);
   $w->bind($path, "<Control-q>" => sub { $p->MainWindow->destroy })
@@ -599,7 +604,9 @@ sub tree {
 		$w->_configure_tree;
 		$w->Busy(-recurse => 1);
 		eval {
-		    $tree->Fill;
+		    $tree->Fill(-fillcb => sub {
+				    $tree->SeePath("file:" . $p->cget(-path)) if $p->cget(-path);
+				});
 		};
 		my $err = $@;
 		$w->Unbusy;
@@ -607,7 +614,6 @@ sub tree {
 		    die $err;
 		}
 	    }
-	    $tree->SeePath("file:" . $p->cget(-path)) if $p->cget(-path);
 	} else {
 	    if ($tree && $tree->manager) {
 		$tree->packForget;
