@@ -1,10 +1,9 @@
 # -*- perl -*-
 
 #
-# $Id: FindPods.pm,v 5.11 2008/02/03 16:10:51 eserte Exp $
 # Author: Slaven Rezic
 #
-# Copyright (C) 2001,2003,2004,2005,2007 Slaven Rezic. All rights reserved.
+# Copyright (C) 2001,2003,2004,2005,2007,2009,2012 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 #
@@ -14,8 +13,6 @@
 
 package Tk::Pod::FindPods;
 
-=encoding iso-8859-2
-
 =head1 NAME
 
 Tk::Pod::FindPods - find Pods installed on the current system
@@ -38,7 +35,7 @@ use vars qw($VERSION @EXPORT_OK $init_done %arch $arch_re);
 
 @EXPORT_OK = qw/%pods $has_cache pod_find/;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 5.11 $ =~ /(\d+)\.(\d+)/);
+$VERSION = '5.15';
 
 BEGIN {  # Make a DEBUG constant very first thing...
   if(defined &DEBUG) {
@@ -87,9 +84,9 @@ be set to force the Pods into a category.
 By default, C<@INC> is scanned for Pods. This can be overwritten by
 the C<-directories> option (specify as an array reference).
 
-If C<-usecache> is specified, then the list of Pods is cached in a
-temporary directory. C<-usecache> is disabled if C<-categorized> is
-not set or C<-directories> is set.
+If C<-usecache> is specified, then the list of Pods is cached (see
+L<cache directory|/Cache directory>). C<-usecache> is disabled if
+C<-categorized> is not set or C<-directories> is set.
 
 =cut
 
@@ -112,17 +109,25 @@ sub pod_find {
 	    my $perllocal_site = File::Spec->catfile($Config{'installsitearch'},'perllocal.pod');
 	    my $perllocal_lib  = File::Spec->catfile($Config{'installarchlib'},'perllocal.pod');
 	    my $cache_file = _cache_file();
-	    if (-r $cache_file &&
-		(-e $perllocal_site && -M $perllocal_site > -M $cache_file) &&
-		(-e $perllocal_lib  && -M $perllocal_lib > -M $cache_file)
-	       ) {
+	VALIDATE_CACHE: {
+		if (!-r $cache_file) {
+		    DEBUG and warn "Cache file $cache_file does not exist or is not readable\n";
+		    last VALIDATE_CACHE;
+		}
+		if (-e $perllocal_site && -M $perllocal_site < -M $cache_file) {
+		    DEBUG and warn "$perllocal_site is more recent than cache file $cache_file\n";
+		    last VALIDATE_CACHE;
+		}
+		if (-e $perllocal_lib  && -M $perllocal_lib < -M $cache_file) {
+		    DEBUG and warn "$perllocal_lib is more recent than cache file $cache_file\n";
+		    last VALIDATE_CACHE;
+		}
+
 		$self->LoadCache;
 		if ($self->{pods}) {
 		    $self->{has_cache} = 1;
 		    return $self->{pods};
 		}
-	    } else {
-		DEBUG and warn "$perllocal_site and/or $perllocal_lib are more recent than cache file $cache_file or cache file does not exist\n";
 	    }
 	}
     }
@@ -383,8 +388,16 @@ sub _cache_file {
 
     my $cache_file_pattern = $ENV{TKPODCACHE};
     if (!defined $cache_file_pattern) {
+	my $cache_root;
+	if ($^O =~ m{^(darwin|MSWin32)} && eval { require File::HomeDir; 1 }) {
+	    $cache_root = File::Spec->catfile(File::HomeDir->my_data, ".tkpod_cache");
+	} elsif ($ENV{HOME} && -d $ENV{HOME}) {
+	    $cache_root = "$ENV{HOME}/.tkpod_cache";
+	} else {
+	    $cache_root = File::Spec->can('tmpdir') ? File::Spec->tmpdir : $ENV{TMPDIR}||"/tmp";
+	}
 	$cache_file_pattern = File::Spec->catfile
-	    (File::Spec->can('tmpdir') ? File::Spec->tmpdir : $ENV{TMPDIR}||"/tmp",
+	    ($cache_root,
 	     join('_', 'pods',"%v","%o","%u")
 	    );
     }
@@ -451,9 +464,9 @@ sub function_pod {
 
 =head2 WriteCache
 
-Write the Pod cache. The cache is written to the temporary directory.
-The file name is constructed from the perl version, operation system
-and user id.
+Write the Pod cache. The cache is written to the L<cache
+directory|/Cache directory>. The file name is constructed from the
+perl version, operation system and user id.
 
 =cut
 
@@ -462,14 +475,23 @@ sub WriteCache {
 
     require Data::Dumper;
 
-    if (!open(CACHE, ">" . _cache_file())) {
-	warn "Can't write to cache file " . _cache_file();
-    } else {
-	my $dd = Data::Dumper->new([$self->{pods}], ['pods']);
-	$dd->Indent(0);
-	print CACHE $dd->Dump;
-	close CACHE;
+    my $cache_dir = dirname _cache_file();
+    if (!-d $cache_dir) {
+	mkdir $cache_dir, 0777
+	    or do {
+		warn "Can't create cache directory $cache_dir: $!";
+		return;
+	    };
     }
+    if (!open(CACHE, ">" . _cache_file())) {
+	warn "Can't write to cache file " . _cache_file() . ": $!";
+	return;
+    }
+
+    my $dd = Data::Dumper->new([$self->{pods}], ['pods']);
+    $dd->Indent(0);
+    print CACHE $dd->Dump;
+    close CACHE;
 }
 
 =head2 LoadCache()
@@ -503,15 +525,27 @@ print Data::Dumper->Dumpxs([Tk::Pod::FindPods->new->pod_find(-categorized => 0,
 
 __END__
 
+=head2 Cache directory
+
+By default the cache file is written to the directory
+F<~/.tkpod_cache> (Unix systems), or the data directory as determined
+by L<File::HomeDir> (Windows, MacOSX). If everything fails, then the
+temporary directory (F</tmp> or the OS equivalent) is used.
+
+If necessary, then the last path component will be created (that is,
+F<.tkpod_cache> will be created if the directory does not exist).
+
+To use another cache directory set the environment variable
+L</TKPODCACHE>.
+
 =head1 ENVIRONMENT
 
 =over
 
 =item TKPODCACHE
 
-Path for the cache file. By default, the cache file is written to the
-temporary directory (F</tmp> or the OS equivalent). The following
-placeholders are recognized:
+Use a custom cache file instead of a file in the L<cache directory|/Cache directory>.
+The following placeholders are recognized:
 
 =over
 
@@ -530,16 +564,15 @@ The user id.
 
 =back
 
-Example for using F</var/tmp> instead of F</tmp> for the cache file
-location (on many systems F</var/tmp> is persistent, unlike F</tmp>):
+Example for using F</some/other/directory> for the cache file location:
 
-	setenv TKPODCACHE /var/tmp/pods_%v_%o_%u
+	TKPODCACHE=/some/other/directory/pods_%v_%o_%u; export TKPODCACHE
 
 or
 
-	TKPODCACHE=/var/tmp/pods_%v_%o_%u; export TKPODCACHE
+	setenv TKPODCACHE /some/other/directory/pods_%v_%o_%u
 
-depending on your shell.
+depending on your shell (sh-like or csh-like).
 
 =back
 
@@ -549,9 +582,9 @@ L<Tk::Tree>.
 
 =head1 AUTHOR
 
-Slaven Rezi <F<slaven@rezic.de>>
+Slaven ReziE<0x0107> <F<slaven@rezic.de>>
 
-Copyright (c) 2001,2003,2004,2005 Slaven Rezic. All rights reserved.
+Copyright (c) 2001,2003,2004,2005,2007,2009 Slaven ReziE<0x0107>. All rights reserved.
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
