@@ -26,7 +26,7 @@ use Tk::Pod::Util qw(is_in_path is_interactive detect_window_manager start_brows
 use vars qw($VERSION @ISA @POD $IDX
 	    @tempfiles @gv_pids $terminal_fallback_warn_shown);
 
-$VERSION = sprintf("%d.%02d", q$Revision: 5.19 $ =~ /(\d+)\.(\d+)/);
+$VERSION = '5.30';
 
 @ISA = qw(Tk::Frame Tk::Pod::SimpleBridge Tk::Pod::Cache);
 
@@ -340,16 +340,21 @@ sub edit
 sub edit_get_linenumber
 {
  my($w) = @_;
- my $linenumber;
+ my $linenumber = $w->get_linenumber;
+ $w->edit(undef, $linenumber);
+}
+
+sub get_linenumber
+{
+ my($w) = @_;
  for my $tag ($w->tagNames('@' . ($w->{MenuX} - $w->rootx) . ',' . ($w->{MenuY} - $w->rooty)))
   {
    if ($tag =~ m{start_line_(\d+)})
     {
-     $linenumber = $1;
-     last;
+     return $1;
     }
   }
- $w->edit(undef, $linenumber);
+ undef;
 }
 
 sub view_source
@@ -364,6 +369,11 @@ sub view_source
 			 -scrollbars => $Tk::platform eq 'MSWin32' ? 'e' : 'w',
 			)->pack(-fill => "both", -expand => 1);
  $more->Load($w->_get_editable_path);
+ my $linenumber = $w->get_linenumber;
+ if (defined $linenumber)
+  {
+   $more->see($linenumber.'.'.0);
+  }
  $more->AddQuitBindings;
  $more->focus;
 }
@@ -444,6 +454,16 @@ sub Populate
     $p_scr->bind('<Shift-Double-1>', sub  { $w->ShiftDoubleClick($_[0]) });
     $p_scr->bind('<Double-2>',       sub  { $w->ShiftDoubleClick($_[0]) });
     $p_scr->bind('<3>',              sub  { $w->PostPopupMenu($p_scr, $w->pointerxy) });
+    $p_scr->bind('<ButtonRelease-2>', [sub {
+					   # A hack solution to prevent from firing this
+					   # event over pod links. See http://wiki.tcl.tk/6101
+					   my($ro,$x,$y) = @_;
+					   if (grep { $_ eq 'pod_link' } $ro->tagNames("\@$x,$y")) {
+					       Tk->break;
+					   } else {
+					       $w->OpenPodBySelection;
+					   }
+				       }, Tk::Ev("x"), Tk::Ev("y")]);
 
     $p->configure(-font => $w->Font(family => 'courier'));
 
@@ -476,7 +496,7 @@ sub Populate
 	  [Button => 'Edit Pod',       -command => sub{ $w->edit_get_linenumber } ],
 	  [Button => 'View source',    -command => sub{ $w->view_source } ],
 	  [Button => 'Copy Pod location', -command => sub { $w->copy_pod_location } ],
-	  [Button => 'Search fulltext',-command => ['SearchFullText', $w]],
+	  [Button => 'Search full text',-command => ['SearchFullText', $w]],
 	  [Separator => ""],
 	  [Cascade => 'Edit',
 	   ($Tk::VERSION > 800.015 && $p->can('EditMenuItems') ? (-menuitems => $p->EditMenuItems) : ()),
@@ -617,58 +637,86 @@ sub Link
 
  if ($sec ne '')
   {
-   # XXX the $start-setting logic doesn't seem to work right
 
-   DEBUG and warn "Looking for section \"$sec\"...\n";
-   DEBUG and warn "Trying a search across Sections entries...\n";
+   my $highlight_match = sub
+    {
+     my $start = shift;
+     my($line) = split(/\./, $start);
+     $w->tag('remove', '_section_mark', qw/0.0 end/);
+     $w->tag('add', '_section_mark',
+	     $line . ".0",
+	     $line . ".0 lineend");
+     $w->yview("_section_mark.first");
+     $w->after(500, [$w, qw/tag remove _section_mark 0.0 end/]);
+    };
 
-   my $start;
+   DEBUG and warn "Looking for section \"$sec\" across Sections entries...\n";
 
-   foreach my $s ( @{$w->{'sections'} || []} ) {
-     if($s->[1] eq $sec) {
+   foreach my $s ( @{$w->{'sections'} || []} )
+    {
+     if($s->[1] eq $sec)
+      {
        DEBUG and warn " $sec is $$s[1] (at $$s[2])\n";
-       $start = $s->[2];
-
+       my $start = $s->[2];
        my($line) = split(/\./, $start);
-       $w->tag('remove', '_section_mark', qw/0.0 end/);
-       $w->tag('add', '_section_mark',
-		  $line-1 . ".0",
-		  $line-1 . ".0 lineend");
-       $w->yview("_section_mark.first");
-       $w->after(500, [$w, qw/tag remove _section_mark 0.0 end/]);
+       $line--; # off by one, why?
+       $highlight_match->("$line.0");
        return;
-     } else {
+      }
+     else
+      {
        DEBUG > 2 and warn " Nope, it's not $$s[1] (at $$s[2])\n";
-     }
-   }
+      }
+    }
 
+   my $start = ($w->tag('nextrange',$sec, '1.0'))[0];
 
-   if( defined $start ) {
+   if (defined $start)
+    {
      DEBUG and warn " Found at $start\n";
-   } else {
-     $start = ($w->tag('nextrange',$sec, '1.0'))[0];
-   }
-
-   my $link = ($man || '') . $sec;
-
-   if( defined $start ) {
-     DEBUG and warn " Found at $start\n";
-   } else {
+     $highlight_match->($start);
+     return;
+    }
+   else
+    {
      DEBUG and warn " Not found so far.  Using a quoted nextrange search...\n";
+     my $link = ($man || '') . $sec;
      $start = ($w->tag('nextrange',"\"$link\"",'1.0'))[0];
-   }
+    }
 
-   if( defined $start ) {
+   if (defined $start)
+    {
      DEBUG and warn " Found at $start\n";
-   } else {
+     $highlight_match->($start);
+     return;
+    }
+   else
+    {
+     DEBUG and warn " Again not found.  Using an exact search at line beginnings...\n";
+     $start = $w->search(qw/-regexp -nocase --/, qr{^\s*\Q$sec}, '1.0');
+    }
+
+   if (defined $start) 
+    {
+     DEBUG and warn " Found at $start\n";
+     $highlight_match->($start);
+     return;
+    }
+   else
+    {
+     DEBUG and warn " Again not found.  Using an exact search...\n";
      $start = $w->search(qw/-exact -nocase --/, $sec, '1.0');
-   }
+    }
 
-
-   unless (defined $start)
+   if (defined $start)
+    {
+     DEBUG and warn " Found at $start\n";
+     $highlight_match->($start);
+     return;
+    }
+   else
     {
      DEBUG and warn " Not found! (\"sec\")\n";
-
      $w->_die_dialog("Section '$sec' not found\n");
     }
    DEBUG and warn "link-zapping to $start linestart\n";
@@ -684,6 +732,25 @@ sub Link
 
 sub Link_url {
     my ($w,$how,$index,$man,$sec) = @_;
+    if (my($lat,$lon) = $man =~ m{^geo:([^,]+),([^,]+)}) {
+        DEBUG and warn "Translate geo URI $man\n";
+        # XXX currently hardcoded to Wikipedia's GeoHack page, maybe make
+        # configurable
+        my $ddd2dms = sub {
+            my($ddd) = @_;
+	    my $north_east = $ddd >= 0;
+	    $ddd = -$ddd if !$north_east;
+	    my $deg = int($ddd);
+	    my $min = ($ddd-$deg)*60;
+	    my $sec = ($min-int($min))*60;
+	    $min = int($min);
+	    (($north_east ? $deg : -$deg), $min, $sec);
+	};
+	my $lat_sgn = $lat < 0 ? do { $lat *= -1; "S" } : "N";
+	my $lon_sgn = $lon < 0 ? do { $lon *= -1; "W" } : "E";
+        $man = "http://toolserver.org/~geohack/geohack.php?params="
+	    . join("_", $ddd2dms->($lat), $lat_sgn, $ddd2dms->($lon), $lon_sgn);
+    }
     DEBUG and warn "Start browser with $man\n";
     start_browser($man);
 }
@@ -835,6 +902,44 @@ sub SearchFullText {
 			},
 			-currentpath => $current_path,
 		       )->pack(-fill=>'both',-expand=>'both');
+	# XXX A very rough solution:
+	$IDX->Button(-text => "Rebuild search index",
+		     -command => sub {
+		      my $pw_bg_msg = "The next dialog will ask for the root password. The search index building will happen in background.";
+		      if (!is_in_path("gksu"))
+		       {
+			if (!is_in_path("xsu"))
+			 {
+			  $w->_error_dialog("gksu or xsu needed to start perlindex");
+			  return;
+			 }
+			$w->_warn_dialog($pw_bg_msg);
+			if (fork == 0)
+		         {
+			  system('xsu',
+				 '--command', 'perlindex -index',
+				 '--username', 'root',
+				 '--title' => 'Rebuild search index',
+				 '--set-display' => $w->screen,
+				);
+			  CORE::exit(0);
+			 }
+		       }
+		      else
+		       {
+			$w->_warn_dialog($pw_bg_msg);
+			if (fork == 0)
+			 {
+			  system('gksu',
+				 '--user', 'root',
+				 #'--description', 'Rebuild search index',
+				 'perlindex -index',
+				);
+			  CORE::exit(0);
+			 }
+		       }
+		     }
+		    )->pack(-fill => 'x');
 	$IDX->Button(-text => "Close",
 		     -command => sub { $IDX->destroy },
 		    )->pack(-fill => 'x');
@@ -1206,6 +1311,18 @@ sub PostPopupMenu {
     $p_scr->PostPopupMenu($X, $Y);
 }
 
+sub OpenPodBySelection {
+    my($w) = @_;
+    my $sel;
+    Tk::catch {
+	$sel = $w->SelectionGet('-selection' => ($Tk::platform eq 'MSWin32'
+						 ? "CLIPBOARD"
+						 : "PRIMARY"));
+    };
+    $sel =~ s{\s}{}g; # no whitespace in Pod names possible
+    $w->configure(-file => $sel);
+}
+
 sub _die_dialog {
     shift->_error_dialog(@_);
     die;
@@ -1415,7 +1532,14 @@ Pod with umlaut: L<ExtUtils::MakeMaker>.
 
 Details:  L<perlpod> or perl, perlfunc.
 
-External links: L<http://www.cpan.org> (URL), L<perl(1)> (man page).
+External links: L<http://www.cpan.org> (URL), L<< URL with link text|http://www.cpan.org >>, L<perl(1)> (man page), L<Berliner Fernsehturm|geo:52.520685,13.409461> (geo: URL)
+
+Links to local sections: L<a section (SYNOPSIS)|/SYNOPSIS>, L<an item
+(-file, currently wrong)|/-file>, L<a working item (auml)|/auml>.
+
+Links to external sections: L<a section (DESCRIPTION in
+perl.pod)|perl/DESCRIPTION>, L<an item (Uncuddled elses in
+perlstyle.pod)|perlstyle/Uncuddled elses>.
 
 Here some code in a as is paragraph
 
@@ -1501,3 +1625,7 @@ terms as Perl itself.
 
 =cut
 
+# Local Variables:
+# mode: cperl
+# cperl-indent-level: 1
+# End:
